/*****************************************************
Date    : 17.10.2013
Author  : Evildesignmail.ru

Chip type               : ATtiny13A
AVR Core Clock frequency: 9,600000 MHz
*****************************************************/
#include <tiny13a.h>


//     .   < 650 .  
#define TIME_LIGHT_ON 25U 

//    100 ,         .
//   60   11.      .   .
//   -       .
//        LP2950    -  .
//       6 . 
#define PHASE_POWER_THR 11

//     -    .
//  -  
#define FOTO_RESISTOR

//   
#define LAMP PORTB.0    

//voltage limit in relative units for six measurements
#define TH_V 250 

volatile unsigned char phase_tick;               //    100   -   
volatile unsigned char max_measured_phase_tick;  //       100     
volatile unsigned char measured_phase_tick;      //        100     
volatile unsigned char light_on;                 // -           0  max_measured_phase_tick
volatile unsigned char phase_locked;             //      
volatile unsigned int num_half_sin;              //           

// Pin change interrupt service routine
//    
interrupt [PC_INT0] void pin_change_isr(void)
{
 //     -   
 if(phase_locked==0) return;
 
#ifdef FOTO_RESISTOR
  if((PINB.2==1)&&(light_on==0)) return; //      .     .
#endif

 if(light_on==0) light_on=1;
 num_half_sin=0;
}

// Timer 0 output compare A interrupt service routine
// Every 100 us start ADC
interrupt [TIM0_COMPA] void timer0_compa_isr(void)
{
 #asm("wdr")
 //   
 phase_tick++;
 if(phase_tick>=max_measured_phase_tick)
   {
    phase_tick=0;
    num_half_sin++;
   }
   
 if(measured_phase_tick<115)measured_phase_tick++;  
 
 
 if(light_on) 
   { 
    //          
    if(phase_tick==0)
      { 
       //      2500   2500*10ms=25 ,  
       if(num_half_sin>=(TIME_LIGHT_ON*100U))
         {
          light_on--;
            
          //         .   .
          if(num_half_sin==(TIME_LIGHT_ON*100U)) 
            {
             GIMSK=(0<<PCIE);
             GIFR=(1<<PCIF);
            }
            
          //        
          if(light_on==0)
            {
             GIMSK=(1<<PCIE);
             GIFR=(1<<PCIF);
            }
         } else
                //      
                if(light_on<max_measured_phase_tick)light_on++;

      } 
      
        
    //        
    if ((phase_tick<=light_on)&&(phase_tick>PHASE_POWER_THR)&&(phase_tick<max_measured_phase_tick-PHASE_POWER_THR)) LAMP=1;
        else     LAMP=0;

   }
         
}

// Analog Comparator interrupt service routine
//       MOSFET
//     
interrupt [ANA_COMP] void ana_comp_isr(void)
{
 if(light_on) 
   {
    LAMP=0; 
    light_on=0;
    num_half_sin=0;
    phase_locked=0;
    GIFR=(1<<PCIF);
   }
}


// ADC interrupt service routine
interrupt [ADC_INT] void adc_isr(void)
{
 static unsigned int adc_prev;
 static unsigned char flag_direction_up; //    
 static unsigned char num_up_samples;    //    
 static unsigned char num_down_samples;  //    
 
 unsigned int adc_data;
 // Read the 10 bits
 // of the AD conversion result
 adc_data=ADCW;
 
 if((adc_data<TH_V)&&(LAMP==0))
   { 
    if (flag_direction_up==0)
       {
        if (adc_data<=adc_prev)  num_down_samples++;       
            else 
                 {                
                  if(num_down_samples>=4)
                    {
                     num_up_samples=1;
                     flag_direction_up=1;
                    } else
                          {
                           num_up_samples=0;
                           num_down_samples=0;
                           flag_direction_up=0;
                          }
                 } 
        } else
              {
               if (adc_data>=adc_prev)
                  {
                   num_up_samples++;         
                   if(num_up_samples==4)
                     {
                      //if the frequency 47,5 to 52,5 Hz, phase locking
                      if((measured_phase_tick>=99)&&(measured_phase_tick<=109))
                        {
                         phase_tick=4;
                         // average                                                                                                       
                         max_measured_phase_tick=(max_measured_phase_tick+(measured_phase_tick-3))>>1;
                         phase_locked=1;
                        }
                      measured_phase_tick=4;
                      num_up_samples=0;
                      num_down_samples=0;
                      flag_direction_up=0; 
                     }
                  } else 
                        { 
                         num_up_samples=0;
                         num_down_samples=0;
                         flag_direction_up=0;
                        }
              }    
   } else 
          {
                  num_up_samples=0;
                  num_down_samples=0;
                  flag_direction_up=0;
          }
 adc_prev=adc_data; 
}



void main(void)
{
// Input/Output Ports initialization
// Port B initialization
// Func5=In Func4=In Func3=In Func2=In Func1=In Func0=Out 
// State5=T State4=T State3=T State2=T State1=T State0=0 
PORTB= 0x00;
DDRB=(1<<DDB0);

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/2k
// Watchdog timeout action: Reset
#pragma optsize-
#asm("wdr")
/* Start timed sequence */
WDTCR |= (1<<WDCE) | (1<<WDE);
/* Set new prescaler(time-out) value = 2K cycles (~16 ms) */
WDTCR = (1<<WDE);
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif



// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 1200,000 kHz
// Mode: CTC top=OCR0A
// OC0A output: Disconnected
// OC0B output: Disconnected
// Timer Period: 0,1 ms
TCCR0A=(0<<COM0A1) | (0<<COM0A0) | (0<<COM0B1) | (0<<COM0B0) | (1<<WGM01) | (0<<WGM00);
TCCR0B=(0<<WGM02) | (0<<CS02) | (1<<CS01) | (0<<CS00);
TCNT0=0x00;
OCR0A=0x77;
OCR0B=0x00;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=(1<<OCIE0A);

// Analog Comparator initialization
// Analog Comparator: On
// The Analog Comparator's positive input is
// connected to the Bandgap Voltage Reference
// Interrupt on Falling Output Edge
ACSR=(1<<ACBG)|(1<<ACIE)|(1<<ACIS1);
ADCSRB=0x00;

// ADC initialization
// ADC Clock frequency: 300,000 kHz
// ADC Bandgap Voltage Reference: On
// ADC Auto Trigger Source: Timer0 Compare Match A
// Only the 10 bits of
// Digital input buffers on ADC0: Off, ADC1: On, ADC2: On, ADC3: Off
DIDR0  = (1<<ADC0D) | (1<<ADC3D) | (1<<AIN1D); 
ADMUX  = (1<<REFS0) | (0<<ADLAR) | (1<<MUX1)  | (1<<MUX0) ;
ADCSRA = (1<<ADEN)  | (0<<ADSC)  | (1<<ADATE) | (1<<ADIF) | (1<<ADIE) | (1<<ADPS2) | (0<<ADPS1) | (1<<ADPS0);
ADCSRB = (1<<ADTS1) | (1<<ADTS0);

// Interrupt on any change on pin PCINT4
PCMSK = (1<<PCINT4);
GIMSK = (1<<PCIE);

ACSR |= (1<<ACI);
TIFR0 = (1<<OCF0B)|(1<<OCF0A)|(1<<TOV0);
GIFR = (1<<PCIF);
MCUCR = 0x00;

max_measured_phase_tick=100; // 100samlpes*100us=10ms     100   

// Global enable interrupts
#asm("sei")

for(;;);
}